from itertools import ifilter

from edge.com import fs, net
from edge.webserver.sysconf import ph
from edge.webserver.sysconf.dgmod import dg
from edge.webserver.sysconf.dgmod.creator import BaseCreator


# noinspection PyUnusedLocal
def lr_get_creators(entry_uimod, all_used_uimods, uimod_to_newconf_map, forced=False):
    """
    :type entry_uimod: ph.UiMod | ph.UiMod2
    :type all_used_uimods: set[ph.UiMod, ph.UiMod2]
    :param uimod_to_newconf_map: Dict[UiMod | UiMod2, Jnode]
    :type uimod_to_newconf_map: dict
    :type forced: bool
    :rtype: list of BaseCreator
    """
    if entry_uimod.type != ph.UI_TYPE_LPR:
        return []

    try:
        cam_uimod = next(ifilter(lambda x: x.type in [ph.UI_TYPE_MV_CAMERA, ph.UI_TYPE_RTSP_CAMERA], all_used_uimods))
        lpr_uimod = next(ifilter(lambda x: x.type == ph.UI_TYPE_LPR, all_used_uimods))
        fusion_uimod = next(ifilter(lambda x: x.type == ph.UI_TYPE_FUSION, all_used_uimods))
        tcp_recipient_uimods = filter(lambda x: x.type == ph.UI_TYPE_TCP_RECIPIENT, all_used_uimods)

        activate = any([
            uimod_to_newconf_map.get(lpr_uimod),
            uimod_to_newconf_map.get(cam_uimod),

            any(map(lambda x: uimod_to_newconf_map.get(x), tcp_recipient_uimods))
        ])

        if forced or activate:
            return [LrCreator(lpr_uimod, cam_uimod, fusion_uimod, tcp_recipient_uimods)]

    except StopIteration:
        pass

    return []


class LrCreator(BaseCreator):
    def __init__(self, lpr_uimod, cam_uimod, fusion_uimod, tcp_recipient_uimods):
        """
        :type lpr_uimod: ph.UiMod
        :type cam_uimod: ph.UiMod
        :type fusion_uimod: ph.UiMod
        :type tcp_recipient_uimods: list of ph.UiMod
        """
        BaseCreator.__init__(self)
        self._lpr_uimod = lpr_uimod
        self._cam_uimod = cam_uimod
        self._fusion_uimod = fusion_uimod
        self._tcp_recipient_uimods = tcp_recipient_uimods

        self._lpr_dgmod = dg.DgMod.create(dg.EDGE_TYPE_LPR, lpr_uimod.pid, lpr_uimod.id)
        self._main_conf = None

    @property
    def dgmod(self):
        """
        :rtype: dg.DgMod
        """
        return self._lpr_dgmod

    def prepare(self, uimod_to_newconf_map):
        """
        :param uimod_to_newconf_map: Dict[UiMod | UiMod2, Jnode]
        :type uimod_to_newconf_map: dict
        """
        spool_path = ph.get_spool_dir(self._lpr_uimod.pid)

        lpr_uiconf = uimod_to_newconf_map.get(self._lpr_uimod) or self._lpr_uimod.load_conf()
        lpr_pri = lpr_uiconf['ro']
        lpr_pub = lpr_uiconf['rw']
        cam_uiconf = uimod_to_newconf_map.get(self._cam_uimod) or self._cam_uimod.load_conf()
        cam_pri = cam_uiconf['ro']

        if self._cam_uimod.type == ph.UI_TYPE_MV_CAMERA:
            if cam_pri['raw-format'] == 'MONO8T':
                add_scene_react_dir = fs.join_path(spool_path, 'mg{}-lr{}-scene'.format(
                    self._cam_uimod.id, self._lpr_uimod.id))
            else:
                add_scene_react_dir = fs.join_path(spool_path, 'rc{}-lr{}-scene'.format(
                    self._cam_uimod.id, self._lpr_uimod.id))

        elif self._cam_uimod.type == ph.UI_TYPE_RTSP_CAMERA:
            add_scene_react_dir = fs.join_path(spool_path, 'rg{}-lr{}-scene'.format(
                self._cam_uimod.id, self._lpr_uimod.id))

        else:
            raise NotImplementedError

        image_wh = ph.CamDgStats.create(self._lpr_uimod.pid).get_stream_wh() or (640, 480)
        scene_event_jnode = [fs.join_path(spool_path, 'lr{}-tr{}-scene'.format(
            self._lpr_uimod.id, self._fusion_uimod.id))]

        # enable gc
        if self._cam_uimod.type == ph.UI_TYPE_MV_CAMERA:
            if cam_uiconf['rw']['videostream']['control']['profile'] == 'auto-70-150':
                scene_event_jnode.append(fs.join_path(spool_path, 'lr{}-gc{}-scene'.format(
                    self._lpr_uimod.id, self._cam_uimod.id)))

        plate_detector_jnode = lpr_pub['plate-detector']
        plate_detector_jnode["extra-ranges-analyse"] = lpr_pri.get('extra-ranges-analyse') or False
        plate_detector_jnode["extra-angle-analyse"] = lpr_pri.get('extra-angle-analyse') or False
        plate_detector_jnode["precise-analyse"] = lpr_pri.get('precise-analyse') or False

        main_conf = {
            "nodes": [
                {
                    "bundle": ["libunity_essentials", "1.0.0"],
                    "type": "essentials",
                    "name": "essentials",
                    "units": [
                        {
                            "type": "api",
                            "name": "api",
                            "conf": {
                                "endpoint": net.get_lr_uds_endpoint(self._lpr_uimod.pid)
                            }
                        },
                        {
                            "type": "counters",
                            "name": "counters",
                            "conf": {
                                "stat-path": fs.join_path(spool_path, 'zabbix', 'lr0'),
                                "stat-tout": 3
                            }
                        }
                    ]
                },
                {
                    "bundle": ["libunity_edge", "1.0.0"],
                    "type": "media-server",
                    "name": "media-server",
                    "units": [
                        {
                            "type": "media",
                            "name": "media",
                            "conf": {
                                "endpoint": add_scene_react_dir
                            }
                        }
                    ]
                },
                {
                    "bundle": ["libunity_edge", "1.0.0"],
                    "type": "event-emitter",
                    "name": "event-emitter",
                    "units": [
                        {
                            "type": "tx-track-bus",
                            "name": "tx-track-bus",
                            "conf": {
                                "store": {
                                    "endpoint": fs.join_path(spool_path, 'lr{0}-lr{0}-store'.format(
                                        self._lpr_uimod.id)),
                                    "min-file-count": 20,
                                    "max-file-count": 25
                                },
                                "track": [fs.join_path(spool_path, 'lr{}-tr{}-track'.format(
                                    self._lpr_uimod.id, self._fusion_uimod.id))]
                            }
                        },
                        {
                            "type": "tx-scene-bus",
                            "name": "tx-scene-bus",
                            "conf": {
                                "scene": scene_event_jnode
                            }
                        }
                    ]
                },
                {
                    "bundle": ["libunity_anpr", "1.0.0"],
                    "type": "anpr",
                    "name": "anpr",
                    "units": [
                        {
                            "type": "anpr",
                            "name": "anpr",
                            "conf": {
                                "engine": {
                                    "log-settings": False,
                                    "license-group": int(lpr_pub['license-group']),
                                    "license-count": lpr_pub['license-count'],
                                    "image-tout": 250,
                                    "plate-detector": plate_detector_jnode,
                                    "plate-recognizer": lpr_pub['plate-recognizer'],
                                    "plate-tracker": {
                                        "lost-tout": lpr_pub['plate-tracker']['lost-tout'],
                                        "size-min": lpr_pri.get('min-track-size', 4)
                                    }
                                },
                                "stream": {
                                    "wh": image_wh,
                                },
                                "postprocess": lpr_pri.get('postprocess') or 1000,
                                "activation": lpr_pri.get('activation') or "image"
                            }
                        }
                    ]
                }
            ],
            "links": [
                ("anpr", "tx-scene-bus"),
                ("anpr", "media"),
                ("anpr", "api"),

                ("counters", "anpr"),
                ("counters", "tx-scene-bus"),
                ("counters", "tx-track-bus")
            ]
        }

        mmr_mode = lpr_pri.get('mmr-mode', None)
        if mmr_mode:
            main_links = main_conf['links']  # type: list
            main_links.append(("anpr", "mmr"))
            main_links.append(("mmr", "tx-track-bus"))
            main_links.append(("mmr", "counters"))

            mmr_jnode = {
                "conf": {
                    "mode": mmr_mode,
                    "min-conf": lpr_pri.get('mmr-min-conf', 50)
                },
                "name": "mmr",
                "type": "mmr"
            }

            nn_jnode = fs.load_json3(ph.NN_CONF_FILEPATH, default=None)
            nn_ro = nn_jnode['ro']  # type: dict
            mmr_jnode['conf'].update(nn_ro['networks'])

            # workaround: xnn should be loaded befoare the anpr.
            main_conf['nodes'].insert(0, {
                "bundle": ["libunity_xnn", "1.0.0"],
                "type": "xnn",
                "name": "xnn",
                "units": [
                    mmr_jnode
                ]
            })

        else:
            main_links = main_conf['links']  # type: list
            main_links.append(("anpr", "tx-track-bus"))

        # create wiegand unit
        wiegand_conf = lpr_pri.get('wiegand-conf', None)
        if wiegand_conf:
            wiegand_jnode = {
                "type": "board",
                "name": "board",
                "conf": wiegand_conf
            }

            main_conf['nodes'].append({
                "bundle": ["libunity_wiegand", "1.0.0"],
                "type": "board",
                "name": "board",
                "units": [
                    wiegand_jnode
                ]
            })

            main_links = main_conf['links']  # type: list
            main_links.append(("anpr", "board"))
            main_links.append(("board", "counters"))

        # create wiegand-udp unit
        wiegand_udp_conf = lpr_pri.get('wiegand-udp-conf', None)
        if wiegand_udp_conf:
            wiegand_udp_jnode = {
                "type": "board-udp",
                "name": "board-udp",
                "conf": wiegand_udp_conf
            }

            main_conf['nodes'].append({
                "bundle": ["libunity_wiegand", "1.0.0"],
                "type": "board-udp",
                "name": "board-udp",
                "units": [
                    wiegand_udp_jnode
                ]
            })

            main_links = main_conf['links']  # type: list
            main_links.append(("anpr", "board-udp"))
            main_links.append(("board-udp", "counters"))

        index = 0
        for tcp_recipient_uimod in self._tcp_recipient_uimods:
            index += 1
            tcp_recipient_conf = uimod_to_newconf_map.get(tcp_recipient_uimod) or tcp_recipient_uimod.load_conf()
            tcp_recipient_pub = tcp_recipient_conf['rw']

            tcp_jnode = {
                "type": "recipient",
                "name": tcp_recipient_uimod.ref,
                "conf": tcp_recipient_pub['conf']
            }

            main_conf['nodes'].append({
                "bundle": ["libunity_recipient", "1.0.0"],
                "type": "vast2",
                "name": "vast2-{}".format(index),
                "units": [
                    tcp_jnode
                ]
            })

            main_links = main_conf['links']  # type: list
            main_links.append(("anpr", tcp_recipient_uimod.ref))
            main_links.append((tcp_recipient_uimod.ref, "counters"))

        self._main_conf = main_conf

    def complete(self):
        lr_dgmod = self._lpr_dgmod

        fs.silent_mkdir_r(fs.join_path(lr_dgmod.conf_dir, 'unit.conf.d'))
        conf_path = fs.join_path(lr_dgmod.conf_dir, 'unit.conf.d', 'main.conf')
        fs.write_json_file(self._main_conf, conf_path, pretty=True)
        lr_dgmod.save_setupstore_conf(active=True)

    def rollback(self):
        pass

    def notify(self):
        """
        :rtype: None
        """
        dg.LifeCycle.restart_dgmod(self.dgmod.type, self.dgmod.id)


class LrDeleter:
    def __init__(self, dgmod):
        """
        :type dgmod: dg.DgMod
        """
        self._dgmod = dgmod

    @classmethod
    def create(cls, entry_uimod):
        """
        :type entry_uimod: ph.UiMod
        :rtype: LrDeleter | None
        """
        if entry_uimod.type == ph.UI_TYPE_LPR:
            dgmod = dg.DgMod.create(dg.EDGE_TYPE_LPR, entry_uimod.pid, entry_uimod.id)
            return cls(dgmod)

    def run(self):
        self._dgmod.uninstall()
